﻿using System;
using System.IO;

namespace Parade.Metadata.Asf
{
	/// <summary>
	/// ASF ファイルの Header Object の基本情報を表します。
	/// 基本情報は Header Object の種別を表す GUID と、格納しているデータのサイズで構成されます。
	/// </summary>
	sealed class HeaderObjectInfo
	{
		/// <summary>
		/// クラスを初期化します。
		/// </summary>
		static HeaderObjectInfo()
		{
			HeaderObjectInfo.ClassByteSize = 24;
		}

		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="stream">Header Object の読み出し位置を指している、ASF ファイルへのストリーム。</param>
		public HeaderObjectInfo( Stream stream )
		{
			this.Guid = stream.ReadGuid();
			this.Size = stream.ReadInt64();
		}

		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="data">バイト配列。</param>
		/// <param name="startIndex">バイト配列から Header Object Info の読み取りを開始する位置</param>
		/// <exception cref="ArgumentException">data または startIndex の内容が不適切。</exception>
		public HeaderObjectInfo( byte[] data, int startIndex )
		{
			// すくなくとも GUID ( 16 バイト ) + サイズ ( 8 バイト ) 分、読み取れなくてはならない
			if( data == null || ( data.Length - startIndex ) < HeaderObjectInfo.ClassByteSize ) { throw new ArgumentException( "data" ); }

			{
				var b = new byte[ 16 ];
				Array.Copy( data, startIndex, b, 0, b.Length );

				this.Guid = new Guid( b );
			}

			this.Size = BitConverter.ToInt64( data, startIndex + 16 );
		}

		/// <summary>
		/// このクラスのサイズ ( バイト単位 ) を取得します。
		/// </summary>
		public static int ClassByteSize { get; private set; }

		/// <summary>
		/// GUID を取得します。
		/// </summary>
		public Guid Guid { get; private set; }

		/// <summary>
		/// サイズを取得します。
		/// </summary>
		public long Size { get; private set; }
	}
}
