﻿using System;
using System.Text;
using System.IO;

namespace Parade.Metadata
{
	/// <summary>
	/// Stream の拡張クラスです。
	/// </summary>
	static class StreamExtentions
	{
		/// <summary>
		/// 真偽値 ( 8 ビット ) を読み取り、ストリームの位置を進めます。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <returns>読み取った値。</returns>
		public static bool ReadBool( this Stream stream )
		{
			return stream.ReadBool( 1 );
		}
		
		/// <summary>
		/// 真偽値を読み取り、ストリームの位置を進めます。
		/// このメソッドでは指定されたバイト数分、ストリームを読み進めますが、
		/// データとして取得される値は、読み取ったバイト配列の先頭部分から必要なデータサイズ分となります。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <param name="count">読み取るバイト数。</param>
		/// <returns>読み取った値。</returns>
		public static bool ReadBool( this Stream stream, int count )
		{
			var b = new byte[ count ];
			stream.Read( b, 0, count );
			return BitConverter.ToBoolean( b, 0 );
		}

		/// <summary>
		/// バイト配列を読み取り、ストリームの位置を進めます。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <param name="count">読み取るバイト数。</param>
		/// <returns>読み取った値。</returns>
		public static byte[] ReadByteArray( this Stream stream, int count )
		{
			var b = new byte[ count ];
			stream.Read( b, 0, count );
			return b;
		}

		/// <summary>
		/// GUID ( 128 ビット ) を読み取り、ストリームの位置を進めます。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <returns>読み取った値。</returns>
		public static Guid ReadGuid( this Stream stream )
		{
			return stream.ReadGuid( 16 );
		}
		
		/// <summary>
		/// GUID を読み取り、ストリームの位置を進めます。
		/// このメソッドでは指定されたバイト数分、ストリームを読み進めますが、
		/// データとして取得される値は、読み取ったバイト配列の先頭部分から必要なデータサイズ分となります。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <param name="count">読み取るバイト数。</param>
		/// <returns>読み取った値。</returns>
		/// <exception cref="ArgumentException">count に指定された数値が 16 バイトを下回っている</exception>
		public static Guid ReadGuid( this Stream stream, int count )
		{
			var b = new byte[ count ];
			stream.Read( b, 0, count );

			if( count == 16 )
			{
				return new Guid( b );
			}
			else if( count > 16 )
			{
				// Guid のコンストラクタに指定するバイト配列は 16 ビットでなければならない
				var data = new byte[ 16 ];
				Array.Copy( b, 0, data, 0, 16 );
				return new Guid( data );
			}
			else
			{
				throw new ArgumentException( "count" );
			}
		}

		/// <summary>
		/// 符号付き 16 ビット整数値を読み取り、ストリームの位置を進めます。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <returns>読み取った値。</returns>
		public static short ReadInt16( this Stream stream )
		{
			return stream.ReadInt16( 2 );
		}

		/// <summary>
		/// 符号付き 16 ビット整数値を読み取り、ストリームの位置を進めます。
		/// このメソッドでは指定されたバイト数分、ストリームを読み進めますが、
		/// データとして取得される値は、読み取ったバイト配列の先頭部分から必要なデータサイズ分となります。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <param name="count">読み取るバイト数。</param>
		/// <returns>読み取った値。</returns>
		public static short ReadInt16( this Stream stream, int count )
		{
			var b = new byte[ count ];
			stream.Read( b, 0, count );
			return BitConverter.ToInt16( b, 0 );
		}

		/// <summary>
		/// 符号付き 32 ビット整数値を読み取り、ストリームの位置を進めます。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <returns>読み取った値。</returns>
		public static int ReadInt32( this Stream stream )
		{
			return stream.ReadInt32( 4 );
		}

		/// <summary>
		/// 符号付き 32 ビット整数値を読み取り、ストリームの位置を進めます。
		/// このメソッドでは指定されたバイト数分、ストリームを読み進めますが、
		/// データとして取得される値は、読み取ったバイト配列の先頭部分から必要なデータサイズ分となります。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <param name="count">読み取るバイト数。</param>
		/// <returns>読み取った値。</returns>
		public static int ReadInt32( this Stream stream, int count )
		{
			var b = new byte[ count ];
			stream.Read( b, 0, count );
			return BitConverter.ToInt32( b, 0 );
		}
	
		/// <summary>
		/// 符号付き 64 ビット整数値を読み取り、ストリームの位置を進めます。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <returns>読み取った値。</returns>
		public static long ReadInt64( this Stream stream )
		{
			return stream.ReadInt64( 8 );
		}
	
		/// <summary>
		/// 符号付き 64 ビット整数値を読み取り、ストリームの位置を進めます。
		/// このメソッドでは指定されたバイト数分、ストリームを読み進めますが、
		/// データとして取得される値は、読み取ったバイト配列の先頭部分から必要なデータサイズ分となります。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <param name="count">読み取るバイト数。</param>
		/// <returns>読み取った値。</returns>
		public static long ReadInt64( this Stream stream, int count )
		{
			var b = new byte[ count ];
			stream.Read( b, 0, count );
			return BitConverter.ToInt64( b, 0 );
		}
		/// <summary>
		/// バイト配列を ASCII 文字列として読み取り、ストリームの位置を進めます。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <param name="count">読み取るバイト数。</param>
		/// <returns>読み取った値。</returns>
		public static string ReadStringAscii( this Stream stream, int count )
		{
			var b = new byte[ count ];
			stream.Read( b, 0, count );

			// 終端の NULL 文字を除去
			var str = Encoding.ASCII.GetString( b );
			if( str[ str.Length - 1 ] == '\0' )
			{
				if( str.Length == 1 ) { return String.Empty; }

				var sb = new StringBuilder( str );
				sb.Remove( sb.Length - 1, 1 );
				str = sb.ToString();
			}

			return str;
		}

		/// <summary>
		/// バイト配列を UTF-16LE 文字列として読み取り、ストリームの位置を進めます。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <param name="count">読み取るバイト数。</param>
		/// <returns>読み取った値。</returns>
		public static string ReadStringUtf16( this Stream stream, int count )
		{
			var b = new byte[ count ];
			stream.Read( b, 0, count );

			// 長さが 2 バイト以下なら UTF-16LE ではないので、空文字として扱う
			if( count < 2 ) { return String.Empty; }

			// 終端の NULL 文字を除去
			var str = Encoding.Unicode.GetString( b, 0, count );
			if( str[ str.Length - 1 ] == '\0' )
			{
				if( str.Length == 1 ) { return String.Empty; }

				var sb = new StringBuilder( str );
				sb.Remove( sb.Length - 1, 1 );
				str = sb.ToString();
			}

			return str;
		}

		/// <summary>
		/// 符号なし 16 ビット整数値を読み取り、ストリームの位置を進めます。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <returns>読み取った値。</returns>
		public static ushort ReadUInt16( this Stream stream )
		{
			return stream.ReadUInt16( 2 );
		}

		/// <summary>
		/// 符号なし 16 ビット整数値を読み取り、ストリームの位置を進めます。
		/// このメソッドでは指定されたバイト数分、ストリームを読み進めますが、
		/// データとして取得される値は、読み取ったバイト配列の先頭部分から必要なデータサイズ分となります。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <param name="count">読み取るバイト数。</param>
		/// <returns>読み取った値。</returns>
		public static ushort ReadUInt16( this Stream stream, int count )
		{
			var b = new byte[ count ];
			stream.Read( b, 0, count );
			return BitConverter.ToUInt16( b, 0 );
		}
		
		/// <summary>
		/// 符号なし 32 ビット整数値を読み取り、ストリームの位置を進めます。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <returns>読み取った値。</returns>
		public static uint ReadUInt32( this Stream stream )
		{
			return stream.ReadUInt32( 4 );
		}

		/// <summary>
		/// 符号なし 32 ビット整数値を読み取り、ストリームの位置を進めます。
		/// このメソッドでは指定されたバイト数分、ストリームを読み進めますが、
		/// データとして取得される値は、読み取ったバイト配列の先頭部分から必要なデータサイズ分となります。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <param name="count">読み取るバイト数。</param>
		/// <returns>読み取った値。</returns>
		public static uint ReadUInt32( this Stream stream, int count )
		{
			var b = new byte[ count ];
			stream.Read( b, 0, count );
			return BitConverter.ToUInt32( b, 0 );
		}

		/// <summary>
		/// 符号なし 64 ビット整数値を読み取り、ストリームの位置を進めます。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <returns>読み取った値。</returns>
		public static ulong ReadUInt64( this Stream stream )
		{
			return stream.ReadUInt64( 8 );
		}

		/// <summary>
		/// 符号なし 64 ビット整数値を読み取り、ストリームの位置を進めます。
		/// このメソッドでは指定されたバイト数分、ストリームを読み進めますが、
		/// データとして取得される値は、読み取ったバイト配列の先頭部分から必要なデータサイズ分となります。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		/// <param name="count">読み取るバイト数。</param>
		/// <returns>読み取った値。</returns>
		public static ulong ReadUInt64( this Stream stream, int count )
		{
			var b = new byte[ count ];
			stream.Read( b, 0, count );
			return BitConverter.ToUInt64( b, 0 );
		}
	}
}
